/************************************************************************************
dijkstrairs
Copyright (c) 2010, International Rescue System Institute
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD license.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/

#ifndef DIJKSTRA_H
#define DIJKSTRA_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>


//-- [ヘッダファイル] ---------------------------------------------------------------------------//
#include <math.h>											// mathヘッダファイル読み込み
#include "dijk.h"											// dijkヘッダファイル読み込み
//-----------------------------------------------------------------------------------------------//


//-- [置換] -------------------------------------------------------------------------------------//
#define STARTENDNO	3										// START/ENDカテゴリ番号
#define PATHNO		4										// PATHカテゴリ番号
#define WAITTIME	1000000										// 起動待ち時間 [us]
#define DATASIZE	512										// 最大データ文字数
#define DEBUG_CONFIGURATION										// DEBUG定義 [デバッグ用]
#define DEBUG_RECV											// DEBUG定義 [デバッグ用]
#define DEBUG_INPORT											// DEBUG定義 [デバッグ用]
#define DEBUG_OUTPORT											// DEBUG定義 [デバッグ用]
#define DEBUG_MESHDATA											// DEBUG定義 [デバッグ用]
#define DEBUG_LINK											// DEBUG定義 [デバッグ用]
#define DEBUG_INITIALIZATION										// DEBUG定義 [デバッグ用]
#define DEBUG_RPATH											// DEBUG定義 [デバッグ用]
#define DEBUG_MESHPATHDATA										// DEBUG定義 [デバッグ用]
#define DEBUG_PATH											// DEBUG定義 [デバッグ用]
#define DEBUG_PATHNEW											// DEBUG定義 [デバッグ用]
//-----------------------------------------------------------------------------------------------//


// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">
#include "intellirobotStub.h"

// </rtc-template>

using namespace RTC;

class Dijkstra
  : public RTC::DataFlowComponentBase
{
 public:
  Dijkstra(RTC::Manager* manager);
  ~Dijkstra();

	// The initialize action (on CREATED->ALIVE transition)
	// formaer rtc_init_entry()
	virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  // virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

	// The activated action (Active state entry action)
	// former rtc_active_entry()
	virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

	// The deactivated action (Active state exit action)
	// former rtc_active_exit()
	virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

	// The execution action that is invoked periodically
	// former rtc_active_do()
	virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);

  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // Configuration variable declaration
  // <rtc-template block="config_declare">
  std::string m_01_location;
  std::string m_02_mapset;
  std::string m_03_type;
  long m_04_originx;
  long m_05_originy;
  unsigned long m_06_sizex;
  unsigned long m_07_sizey;
  unsigned long m_08_pixelsizex;
  unsigned long m_09_pixelsizey;
  std::string m_10_gui;
  std::string m_11_guiaddress;
  std::string m_12_ip;
  std::string m_13_sendport;
  std::string m_14_receiveport;
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  IIS::TimedPosition m_RobotPosition;
  InPort<IIS::TimedPosition> m_RobotPositionIn;
  IIS::TimedPosition m_GoalPosition;
  InPort<IIS::TimedPosition> m_GoalPositionIn;
  TimedMapData m_MeshData;
  InPort<TimedMapData> m_MeshDataIn;
  
  // </rtc-template>

  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  IIS::TimedPath2DSeq m_Path;
  OutPort<IIS::TimedPath2DSeq> m_PathOut;
  TimedMeshOrder m_MeshOrder;
  OutPort<TimedMeshOrder> m_MeshOrderOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

	private:
		int dummy;


		//-- [変数宣言] -----------------------------------------------------------------//

		// プログラム処理
		unsigned long i;									// カウンタ格納変数
		unsigned long j;									// カウンタ格納変数

		// コンフィギュレーション
		char location[DATASIZE];								// 地域名格納配列
		char mapset[DATASIZE];									// 利用者マップセット名格納配列
		char type[DATASIZE];									// データタイプ格納配列
		long int origin_x;									// 対象原点x座標格納変数 [mm]
		long int origin_y;									// 対象原点y座標格納変数 [mm]
		unsigned long int size_x;								// 対象範囲x座標格納変数 [mm]
		unsigned long int size_y;								// 対象範囲y座標格納変数 [mm]
		unsigned long int pixel_size_x;								// ピクセルサイズx座標格納変数 [mm]
		unsigned long int pixel_size_y;								// ピクセルサイズy座標格納変数 [mm]

		// 入力ポート
		unsigned long sec;									// タイムスタンプ格納変数 [sec]
		unsigned long nsec;									// タイムスタンプ格納変数 [nsec]
		long int robot_x;									// ロボットx座標格納変数
		long int robot_y;									// ロボットy座標格納変数
		double robot_theta;									// ロボットtheta格納変数

		unsigned long sec_2;									// タイムスタンプ格納変数 [sec]
		unsigned long nsec_2;									// タイムスタンプ格納変数 [nsec]
		long int goal_x;									// ゴールx座標格納変数
		long int goal_y;									// ゴールy座標格納変数

		unsigned long sec_3;									// タイムスタンプ格納変数 [sec]
		unsigned long nsec_3;									// タイムスタンプ格納変数 [nsec]
		unsigned short mode_3;									// アクセスモード格納変数
		char location_3[DATASIZE];								// 地域名格納配列
		char mapset_3[DATASIZE];								// 利用者マップセット格納配列
		char type_3[DATASIZE];									// データタイプ格納配列
		long origin_x_3;									// 対象原点x座標格納変数 [mm]
		long origin_y_3;									// 対象原点y座標格納変数 [mm]
		unsigned long int size_x_3;								// 対象範囲x座標格納変数 [mm]
		unsigned long int size_y_3;								// 対象範囲y座標格納変数 [mm]
		char explanation_3[DATASIZE];								// 地図説明格納配列
		long map_origin_x_3;									// 地図原点x座標格納変数 [mm]
		long map_origin_y_3;									// 地図原点y座標格納変数 [mm]
		unsigned long map_size_x_3;								// 地図サイズx座標格納変数 [mm]
		unsigned long map_size_y_3;								// 地図サイズy座標格納変数 [mm]
		unsigned long pixel_size_x_3;								// ピクセルサイズx座標格納変数 [mm]
		unsigned long pixel_size_y_3;								// ピクセルサイズy座標格納変数 [mm]
		char category_3[DATASIZE];								// カテゴリデータ格納配列

		// ダイクストラ
		int category_num;									// カテゴリ番号格納変数
		int category_flag;									// フラグ格納変数
		unsigned long *pmesh;									// 地図データ格納配列ポインタ
		unsigned long start_x;									// START地点x座標格納変数
		unsigned long start_y;									// START地点y座標格納変数
		unsigned long end_x;									// END地点x座標格納変数
		unsigned long end_y;									// END地点y座標格納変数
		unsigned long x;									// x座標個数格納変数
		unsigned long y;									// y座標個数格納変数
		unsigned long hazard;									// ハザード番号格納変数
		unsigned long safety;									// セーフティ番号格納変数
		unsigned long nm;									// 要素数格納変数
		unsigned long nstep;									// 反復計算回数格納変数
		unsigned long path_num;									// 経路番号格納変数
		long int is;										// ノード格納変数
		long int path_buf[2];									// 経路一時格納変数
		unsigned long int ipn;									// 要素番号格納変数
		double vmin;										// 最短距離格納変数
		double *pd;										// リンク長格納配列ポインタ
		double *pv;										// 最短距離格納配列ポインタ
		long int *pp;										// 直前ノード格納配列ポインタ
		long int *pm;										// 未求ノード格納配列ポインタ
		long int *prpath;									// 逆ルート格納配列ポインタ
		long int *ppath;									// ルート格納配列ポインタ
		long int *ppath_new;									// ルート格納配列ポインタ [正規化]
		char category_buf[2][DATASIZE];								// カテゴリデータ一時保存配列
		FILE *fd;										// ファイルディスクリプタ格納変数

		// GUIプログラム
		int gui_flag;										// GUI動作フラグ格納変数
		char gui[4];										// GUI動作格納配列
		char gui_addr[256];									// GUIプログラムアドレス格納配列
		char ip[16];										// IPアドレス格納配列
		char send_port[6];									// 送信ポート番号格納配列
		char recv_port[6];									// 受信ポート番号格納配列
		char command[256];									// コマンドライン引数格納配列

		// UDP通信
		char *psend;										// 送信データ格納ポインタ
		char send_data[3 * BUFSIZE];								// 送信データ格納配列
		char recv_data[BUFSIZE];								// 受信データ格納配列
		int send_sock;										// ソケットディスクリプタ格納変数
		int recv_sock;										// ソケットディスクリプタ格納変数
		int recv_len;										// 受信サイズ格納変数
		struct sockaddr_in send_addr;								// アドレス格納構造体
		struct sockaddr_in recv_addr;								// アドレス格納構造体
		struct sigaction saio;									// シグナル受信時動作設定格納構造体
		socklen_t addr_size;									// アドレス格納構造体サイズ格納変数

		//-------------------------------------------------------------------------------//


};


extern "C"
{
  void DijkstraInit(RTC::Manager* manager);
};

#endif // DIJKSTRA_H
